DROP PROC TestSudokuChar81;
go
CREATE PROC TestSudokuChar81
            @Problem char(81)
AS
SET NOCOUNT ON;

CREATE TABLE #Problem
            (Row tinyint NOT NULL CHECK(Row BETWEEN 0 AND 8),
             Col tinyint NOT NULL CHECK(Col BETWEEN 0 AND 8),
             Position AS (Row * 9) + Col + 1 PERSISTED,
             Val char(1) DEFAULT NULL CHECK(Val BETWEEN '1' AND '9'),
             PRIMARY KEY (Row, Col)
            );

WITH         Nums0To8
AS (SELECT   Num
    FROM     Numbers
    WHERE    Num BETWEEN 0 AND 8)
INSERT INTO  #Problem (Row, Col)
SELECT       r.Num, c.Num  -- Implicit conversion will be used
FROM         Nums0To8 AS r
CROSS JOIN   Nums0To8 AS c;

UPDATE       #Problem
SET          Val = SUBSTRING(@Problem, Position, 1)
WHERE        SUBSTRING(@Problem, Position, 1) <> ' ';

/* Uncomment the lines below to test functionality; leave commented for performance comparison. */
/*
-- Display the puzzle
SELECT ' ' + MAX(CASE WHEN Col = 0 THEN Val ELSE ' ' END) + ' ' +
             MAX(CASE WHEN Col = 1 THEN Val ELSE ' ' END) + ' ' +
             MAX(CASE WHEN Col = 2 THEN Val ELSE ' ' END) + ' ' +
            '| ' +
             MAX(CASE WHEN Col = 3 THEN Val ELSE ' ' END) + ' ' +
             MAX(CASE WHEN Col = 4 THEN Val ELSE ' ' END) + ' ' +
             MAX(CASE WHEN Col = 5 THEN Val ELSE ' ' END) + ' ' +
            '| ' +
             MAX(CASE WHEN Col = 6 THEN Val ELSE ' ' END) + ' ' +
             MAX(CASE WHEN Col = 7 THEN Val ELSE ' ' END) + ' ' +
             MAX(CASE WHEN Col = 8 THEN Val ELSE ' ' END) +
             CASE WHEN Row IN (2, 5) THEN CHAR(13) + '-------+-------+-------' ELSE '' END  AS "Starting problem"
FROM         #Problem
GROUP BY     Row
ORDER BY     Row;
*/
go
