DROP PROC TestSudokuCSV;
go
CREATE PROC TestSudokuCSV
            @SudokuCSV varchar(323)   -- Max 81 * row,col,val (81*3=243) + 80 commas (243+80=323)
AS
SET NOCOUNT ON;

CREATE TABLE #Problem
            (Row tinyint NOT NULL CHECK(Row BETWEEN 0 AND 8),
             Col tinyint NOT NULL CHECK(Col BETWEEN 0 AND 8),
             Val char(1) DEFAULT NULL CHECK(Val BETWEEN '1' AND '9'),
             PRIMARY KEY (Row, Col)
            );

WITH         Nums0To8
AS (SELECT   Num
    FROM     Numbers
    WHERE    Num BETWEEN 0 AND 8)
INSERT INTO  #Problem (Row, Col)
SELECT       r.Num, c.Num  -- Implicit conversion will be used
FROM         Nums0To8 AS r
CROSS JOIN   Nums0To8 AS c;

UPDATE       p
SET          Val = s.Val
FROM         #Problem AS p
INNER JOIN  (SELECT SUBSTRING(@SudokuCSV, Num * 4 + 1, 1) - 1 AS Row,
                    SUBSTRING(@SudokuCSV, Num * 4 + 2, 1) - 1 AS Col,
                    SUBSTRING(@SudokuCSV, Num * 4 + 3, 1)     AS Val
             FROM   dbo.Numbers
             WHERE  Num BETWEEN 0 AND LEN(@SudokuCSV)) AS s
      ON     s.Row = p.Row
      AND    s.Col = p.Col;

/* Uncomment the lines below to test functionality; leave commented for performance comparison. */
/*
-- Display the puzzle
SELECT ' ' + MAX(CASE WHEN Col = 0 THEN Val ELSE ' ' END) + ' ' +
             MAX(CASE WHEN Col = 1 THEN Val ELSE ' ' END) + ' ' +
             MAX(CASE WHEN Col = 2 THEN Val ELSE ' ' END) + ' ' +
            '| ' +
             MAX(CASE WHEN Col = 3 THEN Val ELSE ' ' END) + ' ' +
             MAX(CASE WHEN Col = 4 THEN Val ELSE ' ' END) + ' ' +
             MAX(CASE WHEN Col = 5 THEN Val ELSE ' ' END) + ' ' +
            '| ' +
             MAX(CASE WHEN Col = 6 THEN Val ELSE ' ' END) + ' ' +
             MAX(CASE WHEN Col = 7 THEN Val ELSE ' ' END) + ' ' +
             MAX(CASE WHEN Col = 8 THEN Val ELSE ' ' END) +
             CASE WHEN Row IN (2, 5) THEN CHAR(13) + '-------+-------+-------' ELSE '' END  AS "Starting problem"
FROM         #Problem
GROUP BY     Row
ORDER BY     Row;
*/
go
