DROP TABLE TestResults;
go
CREATE TABLE TestResults
      (RunNo int NOT NULL,
       TestGroup varchar(20) NOT NULL,
       TestName varchar(20) NOT NULL,
       Repetitions int NOT NULL,
       Duration int NOT NULL,
       PRIMARY KEY(RunNo, TestGroup, TestName));
go

DECLARE @RunNo int, @cnt smallint, @Repetitions int, @Start datetime;
SET @Repetitions = 10000;
SET @RunNo = 1;

WHILE (1 = 1)
BEGIN;

-- ***********************************
-- First block of tests - using a "hard" Sudoku puzzle from my newspaper
-- ***********************************

SET @Start = CURRENT_TIMESTAMP;
SET @cnt = 0;

WHILE @cnt < @Repetitions
BEGIN;
  SET @cnt = @cnt + 1;
  EXEC TestSudoku81Parms
                   @R1C2 = '7', @R1C5 = '1', @R1C9 = '8',
                   @R2C4 = '8', @R2C5 = '2', @R2C9 = '4',
                   @R3C1 = '1', @R3C6 = '5',
                   @R4C2 = '6', @R4C4 = '1', @R4C6 = '7', @R4C9 = '9',
                   @R5C1 = '7', @R5C3 = '2', @R5C6 = '8', @R5C8 = '1',
                   @R6C4 = '5', @R6C6 = '2', @R6C7 = '7',
                   @R7C2 = '3', @R7C3 = '9', @R7C7 = '5', @R7C9 = '2',
                   @R8C7 = '9', @R8C8 = '4',
                   @R9C5 = '4', @R9C6 = '9';
END;

INSERT INTO TestResults (RunNo, TestGroup, TestName, Repetitions, Duration)
SELECT @RunNo, 'Hard', '81Parms', @Repetitions, DATEDIFF(ms, @Start, CURRENT_TIMESTAMP);


SET @Start = CURRENT_TIMESTAMP;
SET @cnt = 0;

WHILE @cnt < @Repetitions
BEGIN;
  SET @cnt = @cnt + 1;
  EXEC TestSudokuChar81 ' 7  1   8   82   41    5    6 1 7  97 2  8 1    5 27   39   5 2      94     49   ';
END;

INSERT INTO TestResults (RunNo, TestGroup, TestName, Repetitions, Duration)
SELECT @RunNo, 'Hard', 'Char81', @Repetitions, DATEDIFF(ms, @Start, CURRENT_TIMESTAMP);


SET @Start = CURRENT_TIMESTAMP;
SET @cnt = 0;

WHILE @cnt < @Repetitions
BEGIN;
  SET @cnt = @cnt + 1;
  EXEC TestSudokuCSV '127,151,198,248,252,294,311,365,426,441,467,499,517,532,568,581,645,662,677,723,739,775,792,879,884,954,969';
END;

INSERT INTO TestResults (RunNo, TestGroup, TestName, Repetitions, Duration)
SELECT @RunNo, 'Hard', 'CSV', @Repetitions, DATEDIFF(ms, @Start, CURRENT_TIMESTAMP);

-- ***********************************
-- Second block of tests - using an "easy" Sudoku puzzle from a Sudoku computer program
-- ***********************************

SET @Start = CURRENT_TIMESTAMP;
SET @cnt = 0;

WHILE @cnt < @Repetitions
BEGIN;
  SET @cnt = @cnt + 1;
  EXEC TestSudoku81Parms
                   @R1C2 = '7', @R1C6 = '4', @R1C7 = '3', @R1C9 = '2',
                   @R2C7 = '5', @R2C9 = '1',
                   @R3C2 = '3', @R3C3 = '5', @R3C4 = '2', @R3C5 = '7', @R3C8 = '8',
                   @R4C2 = '4', @R4C6 = '5', @R4C7 = '1', @R4C8 = '2',
                   @R5C3 = '9', @R5C4 = '8', @R5C6 = '2', @R5C7 = '7',
                   @R6C2 = '5', @R6C3 = '8', @R6C4 = '7', @R6C8 = '4',
                   @R7C2 = '8', @R7C5 = '3', @R7C6 = '6', @R7C7 = '4', @R7C8 = '5',
                   @R8C1 = '3', @R8C3 = '7',
                   @R9C1 = '5', @R9C3 = '4', @R9C4 = '1', @R9C8 = '6';
END;

INSERT INTO TestResults (RunNo, TestGroup, TestName, Repetitions, Duration)
SELECT @RunNo, 'Easy', '81Parms', @Repetitions, DATEDIFF(ms, @Start, CURRENT_TIMESTAMP);


SET @Start = CURRENT_TIMESTAMP;
SET @cnt = 0;

WHILE @cnt < @Repetitions
BEGIN;
  SET @cnt = @cnt + 1;
  EXEC TestSudokuChar81 ' 7   43 2      5 1 3527  8  4   512   98 27   587   4  8  3645 3 7      5 41   6 ';
END;

INSERT INTO TestResults (RunNo, TestGroup, TestName, Repetitions, Duration)
SELECT @RunNo, 'Easy', 'Char81', @Repetitions, DATEDIFF(ms, @Start, CURRENT_TIMESTAMP);


SET @Start = CURRENT_TIMESTAMP;
SET @cnt = 0;

WHILE @cnt < @Repetitions
BEGIN;
  SET @cnt = @cnt + 1;
  EXEC TestSudokuCSV '127,164,173,192,275,291,323,335,342,357,388,424,465,471,482,539,548,562,577,625,638,647,684,728,753,766,774,785,813,837,915,934,941,986';
END;

INSERT INTO TestResults (RunNo, TestGroup, TestName, Repetitions, Duration)
SELECT @RunNo, 'Easy', 'CSV', @Repetitions, DATEDIFF(ms, @Start, CURRENT_TIMESTAMP);

-- ***********************************
-- Third block of tests - filling only five cells (so not a real Sudoku at all)
-- ***********************************

SET @Start = CURRENT_TIMESTAMP;
SET @cnt = 0;

WHILE @cnt < @Repetitions
BEGIN;
  SET @cnt = @cnt + 1;
  EXEC TestSudoku81Parms @R1C2 = '7', @R2C9 = '1', @R5C6 = '2', @R7C7 = '4', @R9C4 = '1';
END;

INSERT INTO TestResults (RunNo, TestGroup, TestName, Repetitions, Duration)
SELECT @RunNo, 'Empty', '81Parms', @Repetitions, DATEDIFF(ms, @Start, CURRENT_TIMESTAMP);


SET @Start = CURRENT_TIMESTAMP;
SET @cnt = 0;

WHILE @cnt < @Repetitions
BEGIN;
  SET @cnt = @cnt + 1;
  EXEC TestSudokuChar81 ' 7               1                       2                  4              1     ';
END;

INSERT INTO TestResults (RunNo, TestGroup, TestName, Repetitions, Duration)
SELECT @RunNo, 'Empty', 'Char81', @Repetitions, DATEDIFF(ms, @Start, CURRENT_TIMESTAMP);


SET @Start = CURRENT_TIMESTAMP;
SET @cnt = 0;

WHILE @cnt < @Repetitions
BEGIN;
  SET @cnt = @cnt + 1;
  EXEC TestSudokuCSV '127,291,562,774,941';
END;

INSERT INTO TestResults (RunNo, TestGroup, TestName, Repetitions, Duration)
SELECT @RunNo, 'Empty', 'CSV', @Repetitions, DATEDIFF(ms, @Start, CURRENT_TIMESTAMP);

-- ***********************************
-- Fourth block of tests - filling almost all cells (so that would be a REALLY easy Sudoku)
-- ***********************************

SET @Start = CURRENT_TIMESTAMP;
SET @cnt = 0;

WHILE @cnt < @Repetitions
BEGIN;
  SET @cnt = @cnt + 1;
  EXEC TestSudoku81Parms
           @R1C1 = '2',              @R1C3 = '3', @R1C4 = '9', @R1C5 = '1', @R1C6 = '4', @R1C7 = '6', @R1C8 = '5', @R1C9 = '8',
           @R2C1 = '9', @R2C2 = '5', @R2C3 = '6', @R2C4 = '8', @R2C5 = '2', @R2C6 = '3', @R2C7 = '1',              @R2C9 = '4',
           @R3C1 = '1', @R3C2 = '8', @R3C3 = '4', @R3C4 = '6',              @R3C6 = '5', @R3C7 = '2', @R3C8 = '9', @R3C9 = '3',
           @R4C1 = '5', @R4C2 = '6', @R4C3 = '8', @R4C4 = '1', @R4C5 = '3',              @R4C7 = '4', @R4C8 = '2', @R4C9 = '9',
                        @R5C2 = '9', @R5C3 = '2', @R5C4 = '4', @R5C5 = '6', @R5C6 = '8', @R5C7 = '3', @R5C8 = '1', @R5C9 = '5',
           @R6C1 = '3', @R6C2 = '4', @R6C3 = '1', @R6C4 = '5', @R6C5 = '9', @R6C6 = '2',              @R6C8 = '8', @R6C9 = '6',
           @R7C1 = '4', @R7C2 = '3', @R7C3 = '9',              @R7C5 = '8', @R7C6 = '1', @R7C7 = '5', @R7C8 = '6', @R7C9 = '2',
           @R8C1 = '8', @R8C2 = '2',              @R8C4 = '3', @R8C5 = '5', @R8C6 = '6', @R8C7 = '9', @R8C8 = '4', @R8C9 = '1',
           @R9C1 = '6', @R9C2 = '1', @R9C3 = '5', @R9C4 = '2', @R9C5 = '4', @R9C6 = '9', @R9C7 = '8', @R9C8 = '3'             ;
END;

INSERT INTO TestResults (RunNo, TestGroup, TestName, Repetitions, Duration)
SELECT @RunNo, 'Full', '81Parms', @Repetitions, DATEDIFF(ms, @Start, CURRENT_TIMESTAMP);


SET @Start = CURRENT_TIMESTAMP;
SET @cnt = 0;

WHILE @cnt < @Repetitions
BEGIN;
  SET @cnt = @cnt + 1;
  EXEC TestSudokuChar81 '2 39146589568231 41846 529356813 429 92468315341592 86439 8156282 35694161524983 ';
END;

INSERT INTO TestResults (RunNo, TestGroup, TestName, Repetitions, Duration)
SELECT @RunNo, 'Full', 'Char81', @Repetitions, DATEDIFF(ms, @Start, CURRENT_TIMESTAMP);


SET @Start = CURRENT_TIMESTAMP;
SET @cnt = 0;

WHILE @cnt < @Repetitions
BEGIN;
  SET @cnt = @cnt + 1;
  EXEC TestSudokuCSV '112,133,149,151,164,176,185,198,219,225,236,248,252,263,271,294,311,328,334,346,365,372,389,393,415,426,438,441,453,474,482,499,529,532,544,556,568,573,581,595,613,624,631,645,659,662,688,696,714,723,739,758,761,775,786,792,818,822,843,855,866,879,884,891,916,921,935,942,954,969,978,983';
END;

INSERT INTO TestResults (RunNo, TestGroup, TestName, Repetitions, Duration)
SELECT @RunNo, 'Full', 'CSV', @Repetitions, DATEDIFF(ms, @Start, CURRENT_TIMESTAMP);

-- ***********************************
-- All tests done - show intemrediate results, then repeat from start until interrupted
-- ***********************************

PRINT 'After test run ' + CAST(@RunNo AS varchar(5)) + ':';

-- Show average performance measurements after discarding highest and lowest 10%.
WITH RankedResults
AS (SELECT TestGroup,
           TestName,
           CONVERT(FLOAT, Duration) / Repetitions AS AvgDuration,
           ROW_NUMBER () OVER (
               PARTITION BY TestGroup, TestName
               ORDER BY CONVERT(FLOAT, Duration) / Repetitions
           ) AS rn,
           COUNT(*) OVER (
               PARTITION BY TestGroup, TestName
           ) AS cnt
    FROM TestResults)
SELECT
    TestGroup,
    TestName,
    AVG(AvgDuration) AS AvgDur,
    MAX(AvgDuration) AS MaxDur,
    MIN(AvgDuration) AS MinDur,
    (MAX(AvgDuration) - MIN(AvgDuration)) / AVG(AvgDuration) AS MaxDiff
FROM RankedResults
WHERE
    rn BETWEEN FLOOR((cnt + 10) * 0.1)
    AND CEILING(cnt * 0.9)
GROUP BY
    TestGroup,
    TestName
ORDER BY
    TestGroup,
    AvgDur ASC;

SET @RunNo = @RunNo + 1;
END;

go
select * from TestResults;
go
